# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_atyp.ipynb.

# %% auto 0
__all__ = ['FactQ', 'FactRepr']

# %% ../nbs/00_atyp.ipynb 3
from typing import Literal, Optional, TypeAlias

# %% ../nbs/00_atyp.ipynb 5
FactQ: TypeAlias = Optional['fact']
FactRepr: TypeAlias = Literal['abrv', 'name', 'symb']
