# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/02_errs.ipynb.

# %% auto 0
__all__ = ['FactorError']

# %% ../nbs/02_errs.ipynb 3
#| export

# %% ../nbs/02_errs.ipynb 5
class FactorError(KeyError):
    def __init__(self, key: str):  
        super().__init__(f'No SI unit associated with key: {key}')
